/*PASSWORT GENERATOR UND VERWALTER*/

   -- make sure we run this program only, if ooRexx 5.0 or higher (ooRexx 4.2.0 exhibits a runtime error)
parse version "_" v "."
parse source  . . pgm
if v<5 then
do
   parse version v
   .error~say(pp(pgm)" needs ooRexx 5.0 or higher, found:" pp(v))
   .bsf.dialog~messageBox(pp(pgm)" needs ooRexx 5.0 or higher," "0a"x || "found:" pp(v), -
                          "Wrong version of ooRexx", "error")
   exit -1
end
   -- change directory to program location such that relatively addressed resources can be found
call directory filespec('L', pgm)   -- change to the directory where the program resides


var = ""
.environment~my.app=.directory~new  -- directory to contain objects relevant to this application
.my.app~bDebug=.false               /* if set to .true, "put_FXID_objects_into.my.app.rex" will show
                                       all entries in ScriptContext Bindings on the console           */
   -- starting with JavaFX 8u40 true dialogs got introduced; taking advantage of them if running on Java 1.8 or higher

   -- import JavaFX classes that we may use more often
call bsf.import "javafx.fxml.FXMLLoader",                      "fx.FXMLLoader"
call bsf.import "javafx.scene.Scene",                          "fx.Scene"
call bsf.import "javafx.beans.property.SimpleStringProperty",  "fx.SimpleStringProperty"
call bsf.import "javafx.beans.property.SimpleIntegerProperty", "fx.SimpleIntegerProperty"

call bsf.import "javafx.collections.FXCollections",            "fx.FXCollections"
call bsf.import "javafx.stage.Modality",                       "fx.Modality"

   -- create Rexx object that will control the application
rexxApp=.RxMainApplication~new
if arg()>0 then      -- if a command line JSON filename was given, use it to set the attribute
do
   parse arg jsonFile
   rexxApp~pwgFilePath=strip(jsonFile) -- save filePath, if any, in the attribute "personFilePath"
end
.my.app~mainApp=rexxApp        -- store the Rexx MainApp object in .my.app

   -- instantiate the abstract JavaFX class, the abstract "start" method will be served by rexxApp
jRexxApp=BsfCreateRexxProxy(rexxApp, ,"javafx.application.Application")

call sqlite_conn
tmpColl = result
conn = tmpColl
statement = conn~createStatement
query=statement~execute('CREATE TABLE IF NOT EXISTS PWtable (id integer primary key , siteName text, Nutzername text, mail text, passwort text);')


signal on syntax
   -- launch the application, invoke "start" and then stay up until the application closes
jRexxApp~launch(jRexxApp~getClass, .nil)  -- need to use this version of launch in order to work
call sysSleep 0.01                 -- let ooRexx clean up
exit

syntax:
   co=condition("object")
   say ppJavaExceptionChain(co, .true) -- display Java exception chain and stack trace of original Java exception
   say " done. "~center(100, "-")
   exit -1

::requires "BSF.CLS"          -- get Java support
::requires "json-rgf.cls"     -- get JSON support for ooRexx (to load and store person data)
::requires "UNO.CLS"
::requires "database.rex"

/* =================================================================================== */
/* implements the abstract method "start" for the Java class "javafx.application.Application"
   (BSF4ooRexx also supplies another (trailing) slotDir (a Rexx Directory) argument, as "start" is
   invoked from Java).
   It also controls
*/
::class RxMainApplication

   /* for tutorial Part 2: create an ObservableList and create and store Person data in it */
::attribute pwgData

::attribute primaryStage   -- stores the primaryStage supplied via the start method by JavaFX

   -- tutorial, part 5
::attribute pwgFilePath -- .nil or the file path to the file containing the attribute records (in JSON format)


::method init  -- constructor
  expose pwgData prefs pwgFilePath    -- observableArrayList (maintained by TableView)

   -- create and save an ObservableList
  pwgData=.fx.FXCollections~observableArrayList

   -- do we have a file save already, such that a "filePath" preference was set at the user's root node?
  prefs=bsf.loadClass("java.util.prefs.Preferences")~userRoot   -- get the user's root preference node, tutorial part # 5
  pwgFilePath=prefs~get("filePath", .nil) -- get filePath preference, if any

  if pwgFilePath<>.nil then
  do
     if sysFileExists(pwgFilePath) then
     do
        self~loadpwgDataFromFile(pwgfilePath)
        say "loading pwg data from:" pp(pwgfilePath)
     end
     else   -- if file does not exist, do not show it and remove preference
     do
        say "preference address book file" pp(pwgFilePath) "does not exist!"
        pwgFilePath=.nil
        say "-> removing preference" pp("filePath")
        prefs~remove("filePath")
     end
  end
  else   -- temporarily make sure we have (debug) data available to us
  do
     say "no" pp("filePath") "preference found, creating dummy data..."
         -- this creates ooRexx pwg objects that get stored as Java RexxProxy objects, to be used for the TableView
     pwgData~add(.pwg~new("Facebook", "Muster","mail"))
     pwgData~add(.pwg~new("Instagramm", "Mueller", "mail"))

  end


   /* loads the fxml document defining the GUI elements, sets up a scene for it and shows it */
::method start    -- will be invoked by the "launch" method
  expose locale primaryStage pwgFilePath
  use arg primaryStage  -- we get the stage to use for our UI

   -- starting with JavaFX 8u40 true dialogs got introduced; taking advantage of them if running on Java 1.8 or higher
   -- get the JavaFX runtime version (only available when running it), e.g. "8.0.111-b14"

  bAlertsAvailable=(.java.lang.system~getProperty("javafx.runtime.version")>8)
  .my.app~bDialogsAlertsAvailable=bAlertsAvailable  -- determine availability
  if bAlertsAvailable then    -- available since JavaFX 8u40
  do
      call bsf.import "javafx.scene.control.Alert",                  "fx.Alert"
         -- note "AlertType" is public "inner" enum class, i.e. it is defined within the class "Alert", hence
         -- the JavaDocs refer to "AlertType" as "Alert.AlertType": however the Java compiler produces the
         -- class file as "Alert$AlertType"
      call bsf.import "javafx.scene.control.Alert$AlertType",        "fx.Alert.Type"
  end

  if pwgFilePath=.nil then primaryStage~setTitle("Passwort Generator")
                         else primaryStage~title="Passwort Generator" pwgFilePath

   -- create an URL for the FMXLDocument.fxml file (hence the protocol "file:")
  rootLayoutUrl=.bsf~new("java.net.URL", "file:Umgebung_02.fxml")
  rootLayout   =.fx.FXMLLoader~load(rootLayoutUrl) -- load the fxml document
  .rootLayoutController~new(self)   -- create an instance of the controller

  scene=.fx.scene~new(rootLayout)   -- create a scene for our document
  primaryStage~setScene(scene)      -- set the stage to our scene

   -- add application icon
  img=.bsf~new("javafx.scene.image.Image", "file:FotoPW.png")
  primaryStage~getIcons~add(img)

   -- load pwgOverview.fxml, place it into the rootLayout, create the Rexx object controlling the PersonOverview form
  overviewUrl=.bsf~new("java.net.URL", "file:uebersicht.fxml")
  overviewPage=.fx.FXMLLoader~load(overviewUrl)   -- load the fxml document, AnchorPage (root) returned
  .my.app~overviewPage=overviewPage
  rootLayout~setCenter(overviewPage)
  .my.app~pwgOverviewController=.PwgOverviewController~new

  primaryStage~show        -- now show the stage (and thereby our scene)


/* tutorial, step 5: get filePath via Java's Preferences mechanism   */
::method getpwgFilePath
  expose prefs
  filePath=prefs~get("filePath", .nil)
  return filePath


/* tutorial, step 5: set filePath via Java's Preferences mechanism (supplying .nil will remove the preference)   */
::method setpwgFilePath
  expose prefs primaryStage
  use arg filePath

  if file<>.nil then
  do
     prefs~put("filePath", filePath)
     primaryStage~title="PW Generator" filePath
  end
  else
  do
     prefs~remove("filePath")
     primaryStage~title="PW Generator"
  end


/* tutorial, step 5: using JSON instead of XML as file format */
::method loadpwgDataFromFile
  expose pwgData var    -- get access to the ObservableList
  use arg filePath

  s=.stream~new(filepath)~~open("read")   -- open stream
  jsonStr=s~charIn(1, s~query("size"))    -- read entire content
  s~close                                 -- close stream
  pwgall=.Json~new~fromJSON(jsonStr)
  pwgData~clear      -- clear the ObservableList
  do p over pwgall     -- add persons to ObservableList
     pwgData~add(.pwg~new( p["siteName"], p["Nutzername"], p["mail"],p["passwort"]))

  end


 /*tutorial, step 5: using JSON instead of XML as file format*/
::method savepwgDataToFile
  expose pwgData     -- get access to the ObservableList
  use arg filePath

  arr=.array~new
  do p over pwgData  -- iterate over ObservableList
     dir=.directory~new
     dir["siteName"] =p~siteName
     dir["Nutzername"]  =p~Nutzername
     dir["mail"]    =p~mail
     dir["passwort"]    =p~passwort
     --dir["Security"]=p~Security
     arr~append(dir)
  end

  s=.stream~new(filepath)~~open("write replace")   -- open stream for replacement
  jsonStr=s~charout(.json~new~toJson(arr)) -- create and write JSON string to stream
  s~close                              -- close stream
  self~setpwgFilePath(filePath)     -- save this one as preference for next load

--------------------

/* Load the fxml-form, define a stage for it, create an instance of the Rexx class controlling
   the fxml-form.
   Unlike the original tutorial we do not create the dialog and controller over and over again, we
   just cache them using two attributes and reuse them.
*/
::method showPWEditDialog
  expose pedDialog pedController primaryStage
  use arg pwg

  if \var("pedDIALOG") then   -- not yet created, setup PersonEdit data
  do
     -- Load the fxml file and create a new stage for the popup
     pwEditDialogUrl=.bsf~new("java.net.URL", "file:PW_anlegen_01.fxml")
     page  = .fx.FXMLLoader~load(pwEditDialogUrl)
     scene = .fx.Scene~new(page)

     -- AnchorPane page = (AnchorPane) loader.load();
     dialogStage = .bsf~new("javafx.stage.Stage")
     dialogStage~setTitle("Create a new one / Edit your Data")
     dialogStage~initModality(.fx.Modality~WINDOW_MODAL)

     dialogStage~initOwner(primaryStage)
     dialogStage~setScene(scene)

     -- Set the person into the controller
     pedController = .PWEditDialogController~new
     pedController~dialogStage=dialogStage   -- this way the pedController is able to close the stage
  end

  pedController~setpwg(pwg)
  pedController~okClicked=.false          -- make sure we start out with .false

  -- Show the dialog and wait until the user closes it
  dialogStage~showAndWait
  return pedController~OkClicked



/* =================================================================================== */
/* for tutorial part 2: define the PersonOverviewController

   This Rexx class controls the interaction with the fxml form named "PersonOverview.fxml".

*/
::class PwgOverviewController
::attribute pwgTable
::attribute siteName
::attribute Nutzername
::attribute passwort

::method init
  expose pwgTable siteName Nutzername passwort siteNameLabel NutzernameLabel MailLabel passwortLabel -
                     btnNew btnEdit btnDelete btnexport

  -- fetch the directory containing the JavaFX fx:id JavaFX objects and assign them to attributes
  poDir=.my.app~uebersicht.fxml
  pwgTable    =poDir~pwgTable
  siteName=poDir~siteName
  Nutzername=poDir~Nutzername

  /*firstNameColumn=poDir~firstNameColumn
  lastNameColumn =poDir~lastNameColumn*/
  siteName =poDir~siteName
  Nutzername =poDir~Nutzername
  passwort=poDir~passwort
  siteNameLabel=poDir~siteNameLabel
  NutzernameLabel=poDir~NutzernameLabel
  MailLabel=poDir~MailLabel
  passwortLabel=poDir~passwortLabel

   btngen   =poDir~btngen
  btnNew         =poDir~btnNew
  btnEdit        =poDir~btnEdit
  btnDelete      =poDir~btnDelete
   btnexport      =poDir~btnexport

   -- add self as the ChangeListener (we have its method "changed" implemented) and EventHandler (method "handle" implemented)
  rp=BSFCreateRexxProxy(self, ,"javafx.beans.value.ChangeListener", "javafx.event.EventHandler")

  /* set callback for creating TableView rows to which we add us as an event listener to get mouse click events on the rows */
  pwgTable~setOnMouseClicked(rp) -- allows us to get the mouse clicked event

   /* set callback for filling the individual TableView cells */
  /*firstNameColumn*/sitename~setCellValueFactory(BsfCreateRexxProxy(.PropertyValueFactory~new("siteNameProperty"), ,"javafx.util.Callback"))
  /*lastNameColumn*/Nutzername~setCellValueFactory(BsfCreateRexxProxy(.PropertyValueFactory~new("NutzernameProperty"),  ,"javafx.util.Callback"))
  passwort~setCellValueFactory(BsfCreateRexxProxy(.PropertyValueFactory~new("passwortProperty"),  ,"javafx.util.Callback"))

   /* for tutorial part 3: fill-in the Person's details */
  self~showpwgDetails(.nil)   -- clear the labels
   --  auto resize columns
  pwgTable~setColumnResizePolicy(pwgTable~CONSTRAINED_RESIZE_POLICY)    -- TableView constant

   -- add self as the ChangeListener (we have its method "changed" implemented) and EventHandler (method "handle" implemented)
  pwgTable~getSelectionModel ~selectedItemProperty ~addListener(rp)

   -- add us as the event handler to the buttons, the event handler (method "handle") will be able to get at the pressed button
  btnNew   ~setOnAction(rp)
  btnEdit  ~setOnAction(rp)
  btnDelete~setOnAction(rp)
  btnexport~setOnAction(rp)

  self~setItems      -- fill the TableView


::method setItems
  expose pwgTable
  pwgTable~setItems(.my.app~mainApp~pwgData) -- add current personData

/* for tutorial part 3: fill-in the Person's details */
::method showpwgDetails
  expose siteNameLabel NutzernameLabel MailLabel passwortLabel --SecurityLabel firstNameLabel lastNameLabel streetLabel postalCodeLabel cityLabel birthdayLabel
  use arg p

  if p=.nil then  -- reset details
  do
    siteNameLabel  ~setText("")
    NutzernameLabel ~setText("")
    MailLabel     ~setText("")
    passwortLabel     ~setText("")

  end
  else   -- set Pwg's details
  do
     siteNameLabel  ~setText(p~siteName)
     NutzernameLabel ~setText(p~Nutzername)
     MailLabel     ~setText(p~mail)
     passwortLabel     ~setText(p~passwort)

  end


::method handleDeletepwg   /* invoked when the user clicks the delete button   */
  expose pwgTable

  selectedIndex=pwgTable~getSelectionModel~getSelectedIndex



  if selectedIndex >= 0 then
  do
     -- IMPORTANT: we must use bsf.invokeStrict() to become able to give explicitly the argument's type as otherwise
     --            it may be possible that the wrong "remove(Object)" method is used instead of "remove(int)" (both
     --            methods behave differntly):
     p=pwgTable~getItems~bsf.invokeStrict("remove", "int", selectedIndex) -- we want the remove method with the primitive int argument!

        call sqlite_conn
        tmpColl = result
        conn = tmpColl
        statement = conn~createStatement

       abc = selectedIndex +1

        query4 = statement~execute("delete from PWtable where id ='"abc"';")
     -- alternatively, since November 2018 one can use the new BSF.CLS Box.strictArg() routine which makes
     -- sure that the remove method with the 'int' signature gets picked:
     -- personTable~getItems~remove(box.strictArg('int',selectedIndex))  -- 'selectedIndex' will now be used strictly as a primitive Java 'int' value!
  end
  else
  do
      if .my.app~bDialogsAlertsAvailable then
      do
         alert=.fx.alert~new(.fx.Alert.Type~WARNING)   -- create a warning alert

         alert~setTitle("PW Generator Warnung")
         alert~setHeaderText(.nil)
         alert~setContentText("Bitte wähle die Zeile, die du bearbeiten möchtest")
         alert~showAndWait
      end
      else  -- use BSF.CLS' .bsf.dialog utility class
      do
         -- .bsf.dialog is defined in BSF.CLS and uses swing
        .bsf.dialog~messageBox("Bitte wähle die Zeile,  die du bearbeiten möchtest", "No Selection", "warning")
      end
  end


/* for tutorial part 3: fill-in the Person's details */
::method handleNewpwg
  temppwg=.pwg~new


  okClicked = .my.app~mainApp~showPWEditDialog(temppwg)
  if okClicked then
     .my.app~mainApp~pwgData~add(temppwg)


::method handleEditpwg
  expose pwgTable

  selectedpwg = pwgTable~getSelectionModel~getSelectedItem
  if selectedpwg <> .nil then
  do
      pwg=BSFRexxProxy(selectedpwg) -- unbox Rexx object
      okClicked = .my.app~mainApp~showPWEditDialog(pwg)
      if okClicked=.true then
          self~showpwgDetails(pwg)
  end
  else
  do
      if .my.app~bDialogsAlertsAvailable then
      do
         alert=.fx.alert~new(.fx.Alert.Type~WARNING)   -- create a warning alert

         alert~setTitle("PW Generator Information")
         alert~setHeaderText(.nil)
         alert~setContentText("Bitte wähle die Zeile, die du bearbeiten möchtest")
         alert~showAndWait
      end
      else  -- use BSF.CLS' .bsf.dialog utility class
      do
         -- .bsf.dialog is defined in BSF.CLS and uses swing
        .bsf.dialog~messageBox("Bitte wähle die Zeile, die du bearbeiten möchtest", "No Selection", "warning")
      end
  end

::method handleexportpwg
expose pwgtable pwg siteName pwgall var
----------------------------------
xDesktop=uno.createDesktop()
xcl=xDesktop~XComponentLoader
uri="private:factory/scalc"
doc=xcl~loadComponentFromURL(uri,"_blank",0,.uno~noProps)

xSheets=doc~XSpreadSheetDocument~getSheets~XIndexAccess
xSheet =xSheets~getByIndex(0)~XSpreadSheet

call sqlite_conn
tmpColl = result
conn = tmpColl
statement = conn~createStatement

query3= statement~executeQuery('SELECT siteName, Nutzername, mail, passwort from PWtable')

i=2
do while query3~next

    x = query3~getString("siteName")
    call uno.setCell xSheet, 2, i, x
     x = query3~getString("Nutzername")
        call uno.setCell xSheet, 3, i, x
         x = query3~getString("mail")
                call uno.setCell xSheet, 4, i, x
                 x = query3~getString("passwort")
                        call uno.setCell xSheet, 5, i, x
    i = i+1
end


doc~XModifiable~setModified(.false)
call SysSleep 5
doc~XCloseable~close(.false)
----------------------------------------
   -- JavaFX event handlers implemented in Rexx
::method changed unguarded    /* implements the interface "javafx.beans.value.ChangeListener" */
  use arg observable, oldValue, newValue
  self~showpwgDetails(newValue)  -- fill in the labels to show currently selected Person's details

::method handle   /* implements the interface "javafx.event.EventHandler"   */
  expose btnNew btnEdit btnDelete btnexport pwgTable
  use arg event

  tgtObjectName=event~getTarget~objectname
  select
     when tgtObjectName=btnNew~objectName    then self~handleNewpwg
     when tgtObjectName=btnEdit~objectName   then self~handleEditpwg
     when tgtObjectName=btnDelete~objectName then self~handleDeletepwg
     when tgtObjectName=btnexport~objectName then self~handleexportpwg
     otherwise
     do
        if event~getSource~objectName=pwgTable~objectName then -- TableView the source of the event?
        do
            if event~getClickCount>1 then    -- if a double-click, then go into edit record mode
               self~handleEditpwg
        end
        else   -- a truly unknown/unexpected event!
        do
           .error~say(self"::handle, UNKNOWN event:" pp(event~toString))
        end
     end
  end



/* =================================================================================== */
-- implements "R javafx.util.Callback<P,R>(P o) for PropertyValueFactory

/* This class allows instances that remember the message to be sent to person instances to
   return the property of the attribute that should be shown in the table cell.
*/
::class PropertyValueFactory
::method init
  expose  propName   --handler -- name of property getter method
  use strict arg propName -- , handler

::method call
  expose propName   -- handler
  use arg o          -- an observable value for the ooRexx person object boxed in a Java RexxProxy object
  return BsfRexxProxy(o~getValue)~send(propName)


/* =================================================================================== */
/* for tutorial part 3: define the PWEditDialogController */
/*
   This Rexx class controls the interaction with the fxml form named "PWEditDialog.fxml".
*/
::class PWEditDialogController
::attribute okClicked
::attribute dialogStage
::attribute pwg         -- the Person object to edit

::method init
  expose siteNameField NutzernameField mailField passwortField /*SecurityField firstNameField lastNameField streetField postalCodeField cityField birthdayField*/ -
         btngen btnOK btnCancel -
         okClicked --genClicked Frage

  okClicked=.false
--frage
  -- fetch the directory containing the JavaFX fx:id JavaFX objects and assign them to attributes
  pedDir=.my.app~pw_anlegen_01.fxml
    siteNameField =pedDir~siteNameField
    NutzernameField  =pedDir~NutzernameField
    mailField    =pedDir~mailField
    passwortField =pedDir~passwortField

  btnOK          =pedDir~btnOK
  btnCancel      =pedDir~btnCancel
  btngen          =pedDir~btngen

   -- add self as the EventHandler (method "handle" implemented)
  rp=BSFCreateRexxProxy(self, ,"javafx.event.EventHandler")
   -- add us as the event handler to the buttons, the event handler (method "handle") will be able to get at the pressed button
   btngen    ~setOnAction(rp)
  btnOK    ~setOnAction(rp)
  btnCancel~setOnAction(rp)


::method handle      -- method defined "javafx.event.EventHandler"
  expose btnOK btnCancel btngen
  use arg event

  tgtObjectName=event~getTarget~objectname
  select
     when tgtObjectName=btnOK~objectName     then self~handleOK         -- say "BUTTON OK    was the target!"
     when tgtObjectName=btnCancel~objectName then self~handleCancel     -- say "BUTTON CANCEL was the target!"
     when tgtObjectName=btngen~objectName then self~handlegen
     --Frage
     otherwise .error~say("UNKNOWN target:" pp(event~getTarget) pp(event~getTarget~tostring))
  end


::method setpwg
  expose siteNameField NutzernameField mailField passwortField /*SecurityField firstNameField lastNameField streetField postalCodeField cityField birthdayField*/ -
         pwg
  use arg pwg  -- get pwg (assigning it to the attribute)

      siteNameField ~setText(pwg~siteName)
      NutzernameField  ~setText(pwg~Nutzername)
      mailField    ~setText(pwg~mail)
      passwortField    ~setText(pwg~passwort)


::method handleCancel
  expose dialogStage

  dialogStage~hide


::method handleOK
  expose siteNameField NutzernameField mailField passwortField /*SecurityField --PasswortField*/-
         pwg okClicked dialogStage

  if (self~isInputValid) then
  do
      pwg~siteName  =siteNameField~getText
      pwg~Nutzername   = NutzernameField  ~getText
      pwg~mail     = mailField    ~getText
      pwg~passwort     = passwortField    ~getText
      --pwg~Security = SecurityField~getText


        --------------------
        call sqlite_conn
        tmpColl = result
        conn = tmpColl
        statement = conn~createStatement


        input1 = .bsf~new("java.lang.String",pwg~siteName)
         input2 = .bsf~new("java.lang.String",pwg~Nutzername)
         input3 = .bsf~new("java.lang.String",pwg~mail)
         input4 = .bsf~new("java.lang.String",pwg~passwort)--frage
        query4 = statement~execute("Insert INTO PWtable (siteName, Nutzername, mail, passwort) VALUES('"input1~toString"','"input2~toString"','"input3~toString"','"input4~toString"');")


        ---------------------

      okClicked = .true
      dialogStage~hide
  end

::method handlegen
  expose siteNameField NutzernameField mailField passwortField
  --ALGO
  math=bsf.import("java.lang.Math")
  if (self~isInputValid) then
    do
        PW1  =siteNameField~getText
        PW2   = NutzernameField  ~getText
        PW3     = mailField    ~getText
    end

PARSE VAR PW1 PW1before +2
PARSE VAR PW2 PW2before +2

PW1length = length(PW1)
PW2length = length(PW2)
PW3length = length(PW3)

qPW1length =math~pow(PW1length, 2)
qPW2length =math~pow(PW2length, 2)

Sum = qPW1length + qPW2length + PW3length

PWgen=qPW1length || PW1before || qPW2length || PW2before || Sum

passwortField    ~setText(PWgen)



::method isInputValid
  expose siteNameField NutzernameField mailField passwortField -
         dialogStage


  errorMessage = ""
  LF = "0a"x     -- line feed character

  if (siteNameField~getText = .nil | siteNameField~getText~length = 0) then
     errorMessage = LF"No valid Site name"

  if (NutzernameField~getText = .nil | NutzernameField~getText~length = 0) then
     errorMessage ||= LF"No valid Nutzername"

  if (mailField~getText = .nil | mailField~getText~length = 0) then
     errorMessage ||= LF"No valid Mail"

  if (passwortField~getText = .nil | passwortField~getText~length = 0) then
       errorMessage ||= LF"Kein PW berechnet"


  if (errorMessage~length = 0) then
     return .true

  -- .bsf.dialog is defined in BSF.CLS and uses swing
  if .my.app~bDialogsAlertsAvailable then
  do
     alert=.fx.alert~new(.fx.Alert.Type~ERROR)  -- create a warning alert
     alert~setTitle("PW Generator Info")
     alert~setHeaderText("Please correct the listed errors!")
     alert~setContentText("Invalid values:" LF errorMessage)
     alert~showAndWait
  end
  else  -- use BSF.CLS' .bsf.dialog utility class
  do
     -- .bsf.dialog is defined in BSF.CLS and uses swing
    .bsf.dialog~messageBox("Please correct the listed invalid fields:" LF errorMessage, "Invalid Fields", "error")
  end

   return .false


::class pwg

   ----------------------------------------------------------------------------------------
::attribute siteName get
  expose siteName
  return siteName~get

::attribute siteName set
  expose siteName
  use arg val
  return siteName~set(val)

::attribute siteNameProperty get
  expose siteName
  return siteName

   ----------------------------------------------------------------------------------------
::attribute Nutzername get
  expose Nutzername
  return Nutzername~get

::attribute Nutzername set
  expose Nutzername
  use arg val
  return Nutzername~set(val)

::attribute NutzernameProperty get
  expose Nutzername
  return Nutzername

   ----------------------------------------------------------------------------------------
::attribute mail get
  expose mail
  return mail~get

::attribute mail set
  expose mail
  use arg val
  return mail~set(val)

::attribute lastmailProperty get
  expose mail
  return mail

   ----------------------------------------------------------------------------------------

   ::attribute passwort get
     expose passwort
     return passwort~get

   ::attribute passwort set
     expose passwort
     use arg val
     return passwort~set(val)

   ::attribute passwortProperty get
     expose passwort
     return passwort



   ----------------------------------------------------------------------------------------
::method init        -- constructor
  expose siteName Nutzername mail passwort --Security Passwort
  use arg strsiteName="<enter Site Name>", strNutzername="<enter Nutzername>", -
          strmail="enter Mail", /*strSecurity=("1=schwach;2=mittel;3=stark"),*/ -
        strpasswort="Generate or overwrite" --Frage

  siteName= .fx.SimpleStringProperty~new(strsiteName)
  Nutzername = .fx.SimpleStringProperty~new(strNutzername)
  mail    = .fx.SimpleStringProperty~new(strmail)
  --Security= .fx.SimpleStringProperty~new(strSecurity)
  passwort=.fx.SimpleStringProperty~new(strpasswort)



/* =================================================================================== */
/* for tutorial part 5: define the RootLayoutController

   This Rexx class controls the interaction with the fxml form named "RootLayout.fxml".

*/
/* tutorial, part 5 */
::class RootLayoutController

::method init        -- constructor
  expose mainApp -
         menuNew menuOpen  menuSave menuSaveAs menuExit menuHelp

  use arg mainApp -- fetch and save object in attribute

   -- add self as the ChangeListener (we have its method "changed" implemented) and EventHandler (method "handle" implemented)
  rp=BSFCreateRexxProxy(self, ,"javafx.event.EventHandler")

  -- fetch the directory containing the JavaFX fx:id JavaFX objects and assign them us as for event handling (cf. method "handle")
  rlDir=.my.app~umgebung_02.fxml
  menuNew   =rlDir~menuNew   ~~setOnAction(rp)
  menuOpen  =rlDir~menuOpen  ~~setOnAction(rp)
  --menuPrint =rlDir~menuPrint ~~setOnAction(rp)  -- demonstrate JavaFX printing as of JavaFX 8
  menuSave  =rlDir~menuSave  ~~setOnAction(rp)
  menuSaveAs=rlDir~menuSaveAs~~setOnAction(rp)
  menuHelp =rlDir~menuHelp ~~setOnAction(rp)
  menuExit  =rlDir~menuExit  ~~setOnAction(rp)

 /* -- tutorial, part 6
  menuBirthdayStatistics=rlDir~menuBirthdayStatistics ~~setOnAction(rp)*/


::method handle   /* implements the interface "javafx.event"   */
  expose menuNew menuOpen menuSave menuSaveAs menuExit menuHelp mainApp  --menuBirthdayStatistics
  use arg event, slotDir

-- say self"::handle, tid="pp(bsfGetTID())  -- debug statement

  tgtObjectName=event~getTarget~objectname

  select
     when tgtObjectName=menuNew   ~objectName then self~handleNew
     when tgtObjectName=menuOpen  ~objectName then self~handleOpen
     when tgtObjectName=menuSave  ~objectName then self~handleSave
     when tgtObjectName=menuSaveAs~objectName then self~handleSaveAs
     when tgtObjectName=menuHelp ~objectName then self~handleHelp
     when tgtObjectName=menuExit  ~objectName then self~handleExit
      -- tutorial, part 6:
     /*when tgtObjectName=menuBirthdayStatistics~objectName then mainApp~showPersonStatistics
     -- demonstrate JavaFX printing as of JavaFX 8*/
     --when tgtObjectName=menuPrint ~objectName then mainApp~showpwgPrinterDialog

     otherwise .error~say("UNKNOWN target:" pp(event~getTarget) pp(event~getTarget~tostring))
  end


/* create an empty address book */
::method handleNew
  expose mainApp

  mainApp~pwgData~clear
  mainApp~pwgFilePath=.nil
  mainApp~primaryStage~title="PW Generator"

/* load person data from JSON file */
::method handleOpen
  expose mainApp

  fileChooser=.bsf~new("javafx.stage.FileChooser")
   -- crate an array of file extensions (only one in this case)
  jarrExtensions=bsf.createJavaArrayOf("java.lang.String", "*.json")
  extFilter=  .bsf~new("javafx.stage.FileChooser$ExtensionFilter", "JSON files (*.json)", jarrExtensions)
  fileChooser~getExtensionFilters~add(extFilter)
  fileChooser~setInitialDirectory(.bsf~new("java.io.File", "."))  -- set current directory

  -- Show open file dialog
  file = fileChooser~showOpenDialog(mainApp~primaryStage)
  if file<>.nil then
  do
     filePath=file~getPath
     mainApp~loadpwgDataFromFile(filePath)   -- supply the full path, not the Java file object
     mainApp~primaryStage~title="PW Generator" filePath
  end


/* Saves the file to the person file that is currently open. If there is no
 * open file, the "save as" dialog is shown. */
 ::method handleSave
  expose mainApp

  if mainApp~pwgFilePath<>.nil then
     mainApp~savepwgDataToFile(mainApp~pwgFilePath)
  else
     self~handleSaveAs


 /* Opens a FileChooser to let the user select a file to save to. */
::method handleSaveAs
  expose mainApp

  fileChooser=.bsf~new("javafx.stage.FileChooser")
   -- crate an array of file extensions (only one in this case)
  jarrExtensions=bsf.createJavaArrayOf("java.lang.String", "*.json")
  extFilter=  .bsf~new("javafx.stage.FileChooser$ExtensionFilter", "JSON files (*.json)", jarrExtensions)
  fileChooser~getExtensionFilters~add(extFilter)

  filePath=mainApp~pwgFilePath
  if filePath=.nil then
     fileChooser~setInitialDirectory(.bsf~new("java.io.File", "."))  -- set current directory
  else
     fileChooser~setInitialDirectory(.bsf~new("java.io.File", filespec("Directory",filePath)))  -- set directory

  -- Show save file dialog
  file = fileChooser~showSaveDialog(mainApp~primaryStage)
  if file<>.nil then
  do
     filePath=file~getPath
     if filePath~right(5)<>".json" then   -- Make sure it has the correct extension
        filePath||=".json"
     mainApp~savepwgDataToFile( filePath )
  end


/* Opens an about dialog. */
::method handleHelp
  expose mainApp

  --lf="0a"x
    text1= siehe Readme.txt
  if .my.app~bDialogsAlertsAvailable then
  do
     alert=.fx.alert~new(.fx.Alert.Type~information)  -- create an information alert

     alert~setTitle("Passwort Gernerator Hilfe")
     alert~setHeaderText(.nil)
     alert~setContentText(text1)
     alert~showAndWait
  end
  else  -- use BSF.CLS' .bsf.dialog utility class
  do
     -- .bsf.dialog is defined in BSF.CLS and uses swing
    .bsf.dialog~messageBox(text1, "PW Generator", "information")
  end


 /* Closes the application. */
::method handleExit

  bsf.loadClass("javafx.application.Platform")~exit   -- unload JavaFX, but let Rexx continue in main thread
  -- .java.lang.System~exit(0)   -- the Java System class is always available by its environment name
